/*
 * uBrainBasic.c
 *
 */ 

#include "global.h"
#include "uart.h"

#include <avr/pgmspace.h>

const uint8_t ADC_PS_128 = _BV(ADPS2) | _BV(ADPS1) | _BV(ADPS0);
const uint8_t SAMPLES = 16;
const uint16_t THRESHOLD = 514 * 16;

void board_init() {
	// Disable interrupts
	cli();
	
	// Set clock pre-scaler to /1
	CLKPR = _BV(CLKPCE);
	CLKPR = 0x00;
	
	// Enable interrupts
	sei();
	
	DDRA |= _BV(LED_PIN);
}

void led_pwm_init() {
	// Disable timer interrupts
	TIMSK0 = 0x00;
	// Set Timer A for Fast PWM on OC0B
	TCCR0A = _BV(COM0B1) | _BV(WGM01) | _BV(WGM00);

	// Reset timer
	TCNT0 = 0x00;
	// Set compare value
	OCR0B = 0;

	// Set prescaler to /1024 and start pwm
	TCCR0B = _BV(CS02) | _BV(CS00);
}

void hall_init() {
	ADMUX = _BV(MUX1); // VCC Ref, Single ended on ADC2
	ADCSRA = _BV(ADEN) | ADC_PS_128; // enable and set prescaler
}

uint16_t hall_sample() {
	ADCSRA |= _BV(ADSC); // start conversion
	loop_until_bit_is_clear(ADCSRA, ADSC); // wait until conversion is finished
	return ADC;
}


int main(void)
{
	board_init();
	uart_init();
	led_pwm_init();
	hall_init();
	
	static uint32_t hall_sample_agg = 0;

    while(1)
    {
		hall_sample_agg = 0;
		for (uint8_t idx = 0; idx < SAMPLES; idx++) {
			hall_sample_agg += hall_sample();
		}
		OCR0B = (uint8_t)(0x000000FF & (hall_sample_agg >> 4));
		
		printf("%lu\n", hall_sample_agg); 
    }
}